package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.PayDistributionEntity;
import org.springblade.flow.form.model.paydistribution.PayDistributionForm;
import org.springblade.flow.form.model.paydistribution.PayDistributionInfoVO;
import org.springblade.flow.form.service.PayDistributionService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 薪酬发放
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "薪酬发放", value = "PayDistribution")
@RestController
@RequestMapping("/paas/workflow/Form/PayDistribution")
public class PayDistributionController {

	@Autowired
	private PayDistributionService payDistributionService;

	/**
	 * 获取薪酬发放信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取薪酬发放信息")
	@GetMapping("/{id}")
	public ActionResult<PayDistributionInfoVO> info(@PathVariable("id") String id) throws DataException {
		PayDistributionEntity entity = payDistributionService.getInfo(id);
		PayDistributionInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, PayDistributionInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建薪酬发放
	 *
	 * @param payDistributionForm 表单对象
	 * @return
	 */
	@ApiOperation("新建薪酬发放")
	@PostMapping
	public ActionResult create(@RequestBody PayDistributionForm payDistributionForm) throws WorkFlowException {
		PayDistributionEntity entity = JsonUtil.getJsonToBean(payDistributionForm, PayDistributionEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(payDistributionForm.getStatus())) {
			payDistributionService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		payDistributionService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改薪酬发放
	 *
	 * @param payDistributionForm 表单对象
	 * @param id                  主键
	 * @return
	 */
	@ApiOperation("修改薪酬发放")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody PayDistributionForm payDistributionForm, @PathVariable("id") String id)
			throws WorkFlowException {
		PayDistributionEntity entity = JsonUtil.getJsonToBean(payDistributionForm, PayDistributionEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(payDistributionForm.getStatus())) {
			payDistributionService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		payDistributionService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
