package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.PaymentApplyEntity;
import org.springblade.flow.form.model.paymentapply.PaymentApplyForm;
import org.springblade.flow.form.model.paymentapply.PaymentApplyInfoVO;
import org.springblade.flow.form.service.PaymentApplyService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 付款申请单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "付款申请单", value = "PaymentApply")
@RestController
@RequestMapping("/paas/workflow/Form/PaymentApply")
public class PaymentApplyController {

	@Autowired
	private PaymentApplyService paymentApplyService;

	/**
	 * 获取付款申请单信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取付款申请单信息")
	@GetMapping("/{id}")
	public ActionResult<PaymentApplyInfoVO> info(@PathVariable("id") String id) throws DataException {
		PaymentApplyEntity entity = paymentApplyService.getInfo(id);
		PaymentApplyInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, PaymentApplyInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建付款申请单
	 *
	 * @param paymentApplyForm 表单对象
	 * @return
	 */
	@ApiOperation("新建付款申请单")
	@PostMapping
	public ActionResult create(@RequestBody PaymentApplyForm paymentApplyForm) throws WorkFlowException {
		PaymentApplyEntity entity = JsonUtil.getJsonToBean(paymentApplyForm, PaymentApplyEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(paymentApplyForm.getStatus())) {
			paymentApplyService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		paymentApplyService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改付款申请单
	 *
	 * @param paymentApplyForm 表单对象
	 * @param id               主键
	 * @return
	 */
	@ApiOperation("修改付款申请单")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody PaymentApplyForm paymentApplyForm, @PathVariable("id") String id)
			throws WorkFlowException {
		PaymentApplyEntity entity = JsonUtil.getJsonToBean(paymentApplyForm, PaymentApplyEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(paymentApplyForm.getStatus())) {
			paymentApplyService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		paymentApplyService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
