package org.springblade.flow.form.controller;

import java.util.List;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.ProcurementEntryEntity;
import org.springblade.flow.form.entity.ProcurementMaterialEntity;
import org.springblade.flow.form.model.procurementmaterial.ProcurementEntryEntityInfoModel;
import org.springblade.flow.form.model.procurementmaterial.ProcurementMaterialForm;
import org.springblade.flow.form.model.procurementmaterial.ProcurementMaterialInfoVO;
import org.springblade.flow.form.service.ProcurementMaterialService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 采购原材料
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "采购原材料", value = "ProcurementMaterial")
@RestController
@RequestMapping("/paas/workflow/Form/ProcurementMaterial")
public class ProcurementMaterialController {

	@Autowired
	private ProcurementMaterialService procurementMaterialService;

	/**
	 * 获取采购原材料信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取采购原材料信息")
	@GetMapping("/{id}")
	public ActionResult<ProcurementMaterialInfoVO> info(@PathVariable("id") String id) throws DataException {
		ProcurementMaterialEntity entity = procurementMaterialService.getInfo(id);
		List<ProcurementEntryEntity> entityList = procurementMaterialService.getProcurementEntryList(id);
		ProcurementMaterialInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, ProcurementMaterialInfoVO.class);
		vo.setEntryList(JsonUtil.getJsonToList(entityList, ProcurementEntryEntityInfoModel.class));
		return ActionResult.success(vo);
	}

	/**
	 * 新建采购原材料
	 *
	 * @param procurementMaterialForm 表单对象
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("新建采购原材料")
	@PostMapping
	public ActionResult create(@RequestBody ProcurementMaterialForm procurementMaterialForm) throws WorkFlowException {
		ProcurementMaterialEntity procurement = JsonUtil.getJsonToBean(procurementMaterialForm,
				ProcurementMaterialEntity.class);
		List<ProcurementEntryEntity> procurementEntryList = JsonUtil
				.getJsonToList(procurementMaterialForm.getEntryList(), ProcurementEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(procurementMaterialForm.getStatus())) {
			procurementMaterialService.save(procurement.getId(), procurement, procurementEntryList);
			return ActionResult.success("保存成功");
		}
		procurementMaterialService.submit(procurement.getId(), procurement, procurementEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改采购原材料
	 *
	 * @param procurementMaterialForm 表单对象
	 * @param id                      主键
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("修改采购原材料")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody ProcurementMaterialForm procurementMaterialForm,
			@PathVariable("id") String id) throws WorkFlowException {
		ProcurementMaterialEntity procurement = JsonUtil.getJsonToBean(procurementMaterialForm,
				ProcurementMaterialEntity.class);
		List<ProcurementEntryEntity> procurementEntryList = JsonUtil
				.getJsonToList(procurementMaterialForm.getEntryList(), ProcurementEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(procurementMaterialForm.getStatus())) {
			procurementMaterialService.save(id, procurement, procurementEntryList);
			return ActionResult.success("保存成功");
		}
		procurementMaterialService.submit(id, procurement, procurementEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
