package org.springblade.flow.form.controller;

import java.util.List;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.PurchaseListEntity;
import org.springblade.flow.form.entity.PurchaseListEntryEntity;
import org.springblade.flow.form.model.purchaselist.PurchaseListEntryEntityInfoModel;
import org.springblade.flow.form.model.purchaselist.PurchaseListForm;
import org.springblade.flow.form.model.purchaselist.PurchaseListInfoVO;
import org.springblade.flow.form.service.PurchaseListService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 日常物品采购清单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "日常物品采购清单", value = "PurchaseList")
@RestController
@RequestMapping("/paas/workflow/Form/PurchaseList")
public class PurchaseListController {

	@Autowired
	private PurchaseListService purchaseListService;

	/**
	 * 获取日常物品采购清单信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取日常物品采购清单信息")
	@GetMapping("/{id}")
	public ActionResult<PurchaseListInfoVO> info(@PathVariable("id") String id) throws DataException {
		PurchaseListEntity entity = purchaseListService.getInfo(id);
		List<PurchaseListEntryEntity> entityList = purchaseListService.getPurchaseEntryList(id);
		PurchaseListInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, PurchaseListInfoVO.class);
		vo.setEntryList(JsonUtil.getJsonToList(entityList, PurchaseListEntryEntityInfoModel.class));
		return ActionResult.success(vo);
	}

	/**
	 * 新建日常物品采购清单
	 *
	 * @param purchaseListForm 表单对象
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("新建日常物品采购清单")
	@PostMapping
	public ActionResult create(@RequestBody PurchaseListForm purchaseListForm) throws WorkFlowException {
		PurchaseListEntity procurement = JsonUtil.getJsonToBean(purchaseListForm, PurchaseListEntity.class);
		List<PurchaseListEntryEntity> procurementEntryList = JsonUtil.getJsonToList(purchaseListForm.getEntryList(),
				PurchaseListEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(purchaseListForm.getStatus())) {
			purchaseListService.save(procurement.getId(), procurement, procurementEntryList);
			return ActionResult.success("保存成功");
		}
		purchaseListService.submit(procurement.getId(), procurement, procurementEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改日常物品采购清单
	 *
	 * @param purchaseListForm 表单对象
	 * @param id               主键
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("修改日常物品采购清单")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody PurchaseListForm purchaseListForm, @PathVariable("id") String id)
			throws WorkFlowException {
		PurchaseListEntity procurement = JsonUtil.getJsonToBean(purchaseListForm, PurchaseListEntity.class);
		List<PurchaseListEntryEntity> procurementEntryList = JsonUtil.getJsonToList(purchaseListForm.getEntryList(),
				PurchaseListEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(purchaseListForm.getStatus())) {
			purchaseListService.save(id, procurement, procurementEntryList);
			return ActionResult.success("保存成功");
		}
		purchaseListService.submit(id, procurement, procurementEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
