package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.StaffOvertimeEntity;
import org.springblade.flow.form.model.staffovertime.StaffOvertimeForm;
import org.springblade.flow.form.model.staffovertime.StaffOvertimeInfoVO;
import org.springblade.flow.form.service.StaffOvertimeService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 员工加班申请表
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "员工加班申请表", value = "StaffOvertime")
@RestController
@RequestMapping("/paas/workflow/Form/StaffOvertime")
public class StaffOvertimeController {

	@Autowired
	private StaffOvertimeService staffOvertimeService;

	/**
	 * 获取员工加班申请表信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取员工加班申请表信息")
	@GetMapping("/{id}")
	public ActionResult<StaffOvertimeInfoVO> info(@PathVariable("id") String id) throws DataException {
		StaffOvertimeEntity entity = staffOvertimeService.getInfo(id);
		StaffOvertimeInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, StaffOvertimeInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建员工加班申请表
	 *
	 * @param staffOvertimeForm 表单对象
	 * @return
	 */
	@ApiOperation("新建员工加班申请表")
	@PostMapping
	public ActionResult create(@RequestBody StaffOvertimeForm staffOvertimeForm) throws WorkFlowException {
		if (staffOvertimeForm.getStartTime() > staffOvertimeForm.getEndTime()) {
			return ActionResult.fail("结束时间不能小于起始时间");
		}
		StaffOvertimeEntity entity = JsonUtil.getJsonToBean(staffOvertimeForm, StaffOvertimeEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(staffOvertimeForm.getStatus())) {
			staffOvertimeService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		staffOvertimeService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改
	 *
	 * @param staffOvertimeForm 表单对象
	 * @param id                主键
	 * @return
	 */
	@ApiOperation("修改员工加班申请表")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody StaffOvertimeForm staffOvertimeForm, @PathVariable("id") String id)
			throws WorkFlowException {
		if (staffOvertimeForm.getStartTime() > staffOvertimeForm.getEndTime()) {
			return ActionResult.fail("结束时间不能小于起始时间");
		}
		StaffOvertimeEntity entity = JsonUtil.getJsonToBean(staffOvertimeForm, StaffOvertimeEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(staffOvertimeForm.getStatus())) {
			staffOvertimeService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		staffOvertimeService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
