package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.TravelApplyEntity;
import org.springblade.flow.form.model.travelapply.TravelApplyForm;
import org.springblade.flow.form.model.travelapply.TravelApplyInfoVO;
import org.springblade.flow.form.service.TravelApplyService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 出差预支申请单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "出差预支申请单", value = "TravelApply")
@RestController
@RequestMapping("/paas/workflow/Form/TravelApply")
public class TravelApplyController {

	@Autowired
	private TravelApplyService travelApplyService;

	/**
	 * 获取出差预支申请单信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取出差预支申请单信息")
	@GetMapping("/{id}")
	public ActionResult<TravelApplyInfoVO> info(@PathVariable("id") String id) throws DataException {
		TravelApplyEntity entity = travelApplyService.getInfo(id);
		TravelApplyInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, TravelApplyInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建出差预支申请单
	 *
	 * @param travelApplyForm 表单对象
	 * @return
	 */
	@ApiOperation("新建出差预支申请单")
	@PostMapping
	public ActionResult create(@RequestBody TravelApplyForm travelApplyForm) throws WorkFlowException {
		TravelApplyEntity entity = JsonUtil.getJsonToBean(travelApplyForm, TravelApplyEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(travelApplyForm.getStatus())) {
			travelApplyService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		travelApplyService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改出差预支申请单
	 *
	 * @param travelApplyForm 表单对象
	 * @param id              主键
	 * @return
	 */
	@ApiOperation("修改出差预支申请单")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody TravelApplyForm travelApplyForm, @PathVariable("id") String id)
			throws WorkFlowException {
		TravelApplyEntity entity = JsonUtil.getJsonToBean(travelApplyForm, TravelApplyEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(travelApplyForm.getStatus())) {
			travelApplyService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		travelApplyService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
