package org.springblade.flow.form.controller;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.ViolationHandlingEntity;
import org.springblade.flow.form.model.violationhandling.ViolationHandlingForm;
import org.springblade.flow.form.model.violationhandling.ViolationHandlingInfoVO;
import org.springblade.flow.form.service.ViolationHandlingService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 违章处理申请表
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "违章处理申请表", value = "ViolationHandling")
@RestController
@RequestMapping("/paas/workflow/Form/ViolationHandling")
public class ViolationHandlingController {

	@Autowired
	private ViolationHandlingService violationHandlingService;

	/**
	 * 获取违章处理申请表信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取违章处理申请表信息")
	@GetMapping("/{id}")
	public ActionResult<ViolationHandlingInfoVO> info(@PathVariable("id") String id) throws DataException {
		ViolationHandlingEntity entity = violationHandlingService.getInfo(id);
		ViolationHandlingInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, ViolationHandlingInfoVO.class);
		return ActionResult.success(vo);
	}

	/**
	 * 新建违章处理申请表
	 *
	 * @param violationHandlingForm 表单对象
	 * @return
	 */
	@ApiOperation("新建违章处理申请表")
	@PostMapping
	public ActionResult create(@RequestBody ViolationHandlingForm violationHandlingForm) throws WorkFlowException {
		ViolationHandlingEntity entity = JsonUtil.getJsonToBean(violationHandlingForm, ViolationHandlingEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(violationHandlingForm.getStatus())) {
			violationHandlingService.save(entity.getId(), entity);
			return ActionResult.success("保存成功");
		}
		violationHandlingService.submit(entity.getId(), entity);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改违章处理申请表
	 *
	 * @param violationHandlingForm 表单对象
	 * @param id                    主键
	 * @return
	 */
	@ApiOperation("修改违章处理申请表")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody ViolationHandlingForm violationHandlingForm, @PathVariable("id") String id)
			throws WorkFlowException {
		ViolationHandlingEntity entity = JsonUtil.getJsonToBean(violationHandlingForm, ViolationHandlingEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(violationHandlingForm.getStatus())) {
			violationHandlingService.save(id, entity);
			return ActionResult.success("保存成功");
		}
		violationHandlingService.submit(id, entity);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
