package org.springblade.flow.form.controller;

import java.util.List;

import org.springblade.flow.base.ActionResult;
import org.springblade.flow.database.exception.DataException;
import org.springblade.flow.engine.enums.FlowStatusEnum;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.WarehouseEntryEntity;
import org.springblade.flow.form.entity.WarehouseReceiptEntity;
import org.springblade.flow.form.model.warehousereceipt.WarehouseReceiptEntityInfoModel;
import org.springblade.flow.form.model.warehousereceipt.WarehouseReceiptForm;
import org.springblade.flow.form.model.warehousereceipt.WarehouseReceiptInfoVO;
import org.springblade.flow.form.service.WarehouseReceiptService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.JsonUtilEx;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 入库申请单
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Api(tags = "入库申请单", value = "WarehouseReceipt")
@RestController
@RequestMapping("/paas/workflow/Form/WarehouseReceipt")
public class WarehouseReceiptController {

	@Autowired
	private WarehouseReceiptService warehouseReceiptService;

	/**
	 * 获取入库申请单信息
	 *
	 * @param id 主键值
	 * @return
	 */
	@ApiOperation("获取入库申请单信息")
	@GetMapping("/{id}")
	public ActionResult<WarehouseReceiptInfoVO> info(@PathVariable("id") String id) throws DataException {
		WarehouseReceiptEntity entity = warehouseReceiptService.getInfo(id);
		List<WarehouseEntryEntity> entityList = warehouseReceiptService.getWarehouseEntryList(id);
		WarehouseReceiptInfoVO vo = JsonUtilEx.getJsonToBeanEx(entity, WarehouseReceiptInfoVO.class);
		vo.setEntryList(JsonUtil.getJsonToList(entityList, WarehouseReceiptEntityInfoModel.class));
		return ActionResult.success(vo);
	}

	/**
	 * 新建入库申请单
	 *
	 * @param warehouseReceiptForm 表单对象
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("新建入库申请单")
	@PostMapping
	public ActionResult create(@RequestBody WarehouseReceiptForm warehouseReceiptForm) throws WorkFlowException {
		WarehouseReceiptEntity warehouse = JsonUtil.getJsonToBean(warehouseReceiptForm, WarehouseReceiptEntity.class);
		List<WarehouseEntryEntity> warehouseEntryList = JsonUtil.getJsonToList(warehouseReceiptForm.getEntryList(),
				WarehouseEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(warehouseReceiptForm.getStatus())) {
			warehouseReceiptService.save(warehouse.getId(), warehouse, warehouseEntryList);
			return ActionResult.success("保存成功");
		}
		warehouseReceiptService.submit(warehouse.getId(), warehouse, warehouseEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}

	/**
	 * 修改入库申请单
	 *
	 * @param warehouseReceiptForm 表单对象
	 * @param id                   主键
	 * @return
	 * @throws WorkFlowException
	 */
	@ApiOperation("修改入库申请单")
	@PutMapping("/{id}")
	public ActionResult update(@RequestBody WarehouseReceiptForm warehouseReceiptForm, @PathVariable("id") String id)
			throws WorkFlowException {
		WarehouseReceiptEntity warehouse = JsonUtil.getJsonToBean(warehouseReceiptForm, WarehouseReceiptEntity.class);
		List<WarehouseEntryEntity> warehouseEntryList = JsonUtil.getJsonToList(warehouseReceiptForm.getEntryList(),
				WarehouseEntryEntity.class);
		if (FlowStatusEnum.save.getMessage().equals(warehouseReceiptForm.getStatus())) {
			warehouseReceiptService.save(id, warehouse, warehouseEntryList);
			return ActionResult.success("保存成功");
		}
		warehouseReceiptService.submit(id, warehouse, warehouseEntryList);
		return ActionResult.success("提交成功，请耐心等待");
	}
}
