package org.springblade.flow.form.service.impl;

import org.springblade.flow.base.service.BillRuleService;
import org.springblade.flow.engine.util.ModelUtil;
import org.springblade.flow.exception.WorkFlowException;
import org.springblade.flow.form.entity.PayDistributionEntity;
import org.springblade.flow.form.mapper.PayDistributionMapper;
import org.springblade.flow.form.model.paydistribution.PayDistributionForm;
import org.springblade.flow.form.service.PayDistributionService;
import org.springblade.flow.util.JsonUtil;
import org.springblade.flow.util.RandomUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 薪酬发放
 *
 * @author JNPF开发平台组
 * @version V3.1.0
 * @copyright 引迈信息技术有限公司
 * @date 2019年9月29日 上午9:18
 */
@Service
public class PayDistributionServiceImpl extends ServiceImpl<PayDistributionMapper, PayDistributionEntity>
		implements PayDistributionService {

	@Autowired
	private BillRuleService billRuleService;

	@Override
	public PayDistributionEntity getInfo(String id) {
		QueryWrapper<PayDistributionEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(PayDistributionEntity::getId, id);
		return this.getOne(queryWrapper);
	}

	@Override
	@Transactional(rollbackFor = WorkFlowException.class)
	public void save(String id, PayDistributionEntity entity) throws WorkFlowException {
		// 表单信息
		if (id == null) {
			entity.setId(RandomUtil.uuId());
			this.save(entity);
			billRuleService.useBillNumber("WF_PayDistributionNo");
		} else {
			entity.setId(id);
			this.updateById(entity);
		}
		// 流程信息
		ModelUtil.save(id, entity.getFlowId(), entity.getId(), entity.getFlowTitle(), entity.getFlowUrgent(),
				entity.getBillNo(), entity);
	}

	@Override
	@Transactional(rollbackFor = WorkFlowException.class)
	public void submit(String id, PayDistributionEntity entity) throws WorkFlowException {
		// 表单信息
		if (id == null) {
			entity.setId(RandomUtil.uuId());
			this.save(entity);
			billRuleService.useBillNumber("WF_PayDistributionNo");
		} else {
			entity.setId(id);
			this.updateById(entity);
		}
		// 流程信息
		ModelUtil.submit(id, entity.getFlowId(), entity.getId(), entity.getFlowTitle(), entity.getFlowUrgent(),
				entity.getBillNo(), entity, null);
	}

	@Override
	public void data(String id, String data) {
		PayDistributionForm payDistributionForm = JsonUtil.getJsonToBean(data, PayDistributionForm.class);
		PayDistributionEntity entity = JsonUtil.getJsonToBean(payDistributionForm, PayDistributionEntity.class);
		entity.setId(id);
		this.saveOrUpdate(entity);
	}
}
